﻿using ProjectAirPlane.Entites;

namespace ProjectAirPlane.Drawnings;

/// <summary>
/// Класс, отвечающий за прорисовку и перемещение объекта-сущности
/// </summary>
public class DrawningAirPlane : DrawningPlane
{

    /// <summary>
    /// Конструктор
    /// </summary>
    /// <param name="speed">Скорость</param>
    /// <param name="weight">Вес</param>
    /// <param name="bodyColor">Основной цвет</param>
    /// <param name="additionalColor">Дополнительный цвет</param>
    /// <param name="radar">Признак наличия обвеса</param>
    /// <param name="dopBak">Признак наличия антикрыла</param>
    /// <param name="chassi">Признак наличия гоночной полосы</param>
    public DrawningAirPlane(int speed, double weight, Color bodyColor, Color additionalColor, bool radar, bool dopBak, bool chassi) : base(195, 70)
    {
        EntityPlane = new EntityAirPlane(speed, weight, bodyColor, additionalColor, radar, dopBak, chassi);


    }


    public override void DrawTransport(Graphics g)
    {
        if (EntityPlane == null || EntityPlane is not EntityAirPlane airPlane || !_startPosX.HasValue || !_startPosY.HasValue)
        {
            return;
        }
        Pen pen = new(Color.Black);
        Brush additionalBrush = new SolidBrush(EntityPlane.BodyColor);

        if (airPlane.Chassi)
        {
            //шасси
            g.DrawLine(pen, _startPosX.Value + 50, _startPosY.Value + 55, _startPosX.Value + 50, _startPosY.Value + 70);
            g.DrawLine(pen, _startPosX.Value + 150, _startPosY.Value + 55, _startPosX.Value + 150, _startPosY.Value + 70);
            g.FillEllipse(additionalBrush, _startPosX.Value + 40, _startPosY.Value + 65, 10, 10);
            g.FillEllipse(additionalBrush, _startPosX.Value + 50, _startPosY.Value + 65, 10, 10);
            g.FillEllipse(additionalBrush, _startPosX.Value + 145, _startPosY.Value + 65, 10, 10);

        }


        base.DrawTransport(g);



        if (airPlane.DopBak)
        {
            //бак
            g.FillEllipse(additionalBrush, _startPosX.Value, _startPosY.Value + 45, 40, 20);
            g.DrawEllipse(pen, _startPosX.Value, _startPosY.Value + 45, 40, 20);

        }

        if (airPlane.Radar)
        {
            //радар
            g.DrawLine(pen, _startPosX.Value + 60, _startPosY.Value + 25, _startPosX.Value + 60, _startPosY.Value + 15);
            g.DrawLine(pen, _startPosX.Value + 60, _startPosY.Value + 15, _startPosX.Value + 67, _startPosY.Value + 11);
            Point point7 = new Point(_startPosX.Value + 60, _startPosY.Value + 15);
            Point point8 = new Point(_startPosX.Value + 60, _startPosY.Value + 5);
            Point point9 = new Point(_startPosX.Value + 70, _startPosY.Value + 25);
            Point[] curvePoints3 =
             {
                 point7,
                 point8,
                 point9,
             };
            g.FillPolygon(additionalBrush, curvePoints3);

        }

    }
}


