﻿using ProjectAirPlane.Entites;

namespace ProjectAirPlane.Drawnings;

public class DrawningPlane
{
    /// <summary>
    /// Класс-сущность
    /// </summary>
    public EntityPlane? EntityPlane { get; protected set; }

    /// <summary>
    /// Ширина окна
    /// </summary>
    private int? _pictureWidth;

    /// <summary>
    /// Высота окна
    /// </summary>
    private int? _pictureHeight;

    /// <summary>
    /// Левая координата прорисовки автомобиля
    /// </summary>
    protected int? _startPosX;

    /// <summary>
    /// Верхняя кооридната прорисовки автомобиля
    /// </summary>
    protected int? _startPosY;

    /// <summary>
    /// Ширина прорисовки самолёта
    /// </summary>
    private readonly int _drawningAirPlaneWidth = 195;

    /// <summary>
    /// Высота прорисовки самолёта
    /// </summary>
    private readonly int _drawningAirPlaneHeight = 70;

    /// <summary>
	/// Координата X объекта
	/// </summary>
	public int? GetPosX => _startPosX;

    /// <summary>
    /// Координата Y объекта
    /// </summary>
    public int? GetPosY => _startPosY;

    /// <summary>
    /// Ширина объекта
    /// </summary>
    public int GetWidth => _drawningAirPlaneWidth;

    /// <summary>
    /// Высота объекта
    /// </summary>
    public int GetHeight => _drawningAirPlaneHeight;

    /// <summary>
    /// Пустой конструктор
    /// </summary>
    private DrawningPlane()
    {
        _pictureWidth = null;
        _pictureHeight = null;
        _startPosX = null;
        _startPosY = null;
    }

    /// <summary>
    /// Конструктор
    /// </summary>
    /// <param name="speed">Скорость</param>
    /// <param name="weight">Вес</param>
    /// <param name="bodyColor">Основной цвет</param>

    public DrawningPlane(int speed, double weight, Color bodyColor) : this()
    {

        EntityPlane = new EntityPlane(speed, weight, bodyColor);

    }

    /// <summary>
    /// Конструктор для наследования
    /// </summary>
    /// <param name="drawningAirPlaneWidth">Ширина прорисовки самолёта </param>
    /// <param name="drawningAirPlaneHeight">Высота прорисовки самолёта</param>


    protected DrawningPlane(int drawningPlaneWidth, int drawningPlaneHeight) : this()
    {
        _drawningAirPlaneWidth = drawningPlaneWidth;
        _drawningAirPlaneHeight = drawningPlaneHeight;

    }


    /// <summary>
    /// Установка границ поля
    /// </summary>
    /// <param name="width">Ширина поля</param>
    /// <param name="height">Высота поля</param>
    /// <returns>true - границы заданы, false - проверка не пройдена, нельзя разместить объект в этих размерах</returns>
    public bool SetPictureSize(int width, int height)
    {
        // TODO проверка, что объект "влезает" в размеры поля

        if (_drawningAirPlaneWidth > width || _drawningAirPlaneHeight > height)
        {
            return false;
        }

        _pictureWidth = width;
        _pictureHeight = height;
        if (_startPosX.HasValue || _startPosY.HasValue)
        {
            if (_startPosX + _drawningAirPlaneWidth > _pictureWidth)
            {
                _startPosX = _pictureWidth - _drawningAirPlaneWidth;
            }
            else if (_startPosX < 0) _startPosX = 0;
            if (_startPosY + _drawningAirPlaneHeight > _pictureHeight)
            {
                _startPosY = _pictureHeight - _drawningAirPlaneHeight;
            }
            else if (_startPosY < 0) _startPosY = 0;
        }
        return true;
    }

    /// <summary>
    /// Установка позиции
    /// </summary>
    /// <param name="x">Координата X</param>
    /// <param name="y">Координата Y</param>
    public void SetPosition(int x, int y)
    {
        if (!_pictureHeight.HasValue || !_pictureWidth.HasValue)
        {
            return;
        }

        // TODO если при установке объекта в эти координаты, он будет "выходить" за границы формы

        if (x + _drawningAirPlaneWidth >= _pictureWidth || y + _drawningAirPlaneHeight >= _pictureHeight)
        {
            x = y = 10;
            _startPosX = x;
            _startPosY = y;
        }
        // то надо изменить координаты, чтобы он оставался в этих границах
        _startPosX = x;
        _startPosY = y;
    }

    /// <summary>
    /// Изменение направления перемещения
    /// </summary>
    /// <param name="direction">Направление</param>
    /// <returns>true - перемещене выполнено, false - перемещение невозможно</returns>
    public bool MoveTransport(DirectionType direction)
    {
        if (EntityPlane == null || !_startPosX.HasValue || !_startPosY.HasValue)
        {
            return false;
        }

        switch (direction)
        {
            //влево
            case DirectionType.Left:
                if (_startPosX.Value - EntityPlane.Step > 0)
                {
                    _startPosX -= (int)EntityPlane.Step;
                }
                return true;
            //вверх
            case DirectionType.Up:
                if (_startPosY.Value - EntityPlane.Step > 0)
                {
                    _startPosY -= (int)EntityPlane.Step;
                }
                return true;
            // вправо
            case DirectionType.Right:
                //TODO прописать логику сдвига вправо
                if (_startPosX + EntityPlane.Step + _drawningAirPlaneWidth < _pictureWidth)
                {
                    _startPosX += (int)EntityPlane.Step;
                }
                return true;
            //вниз
            case DirectionType.Down:
                //TODO прописать логику сдвига ввниз
                if (_startPosY + EntityPlane.Step + _drawningAirPlaneHeight < _pictureHeight)
                {
                    _startPosY += (int)EntityPlane.Step;
                }
                return true;
            default:
                return false;
        }
    }

    /// <summary>
    /// Прорисовка объекта
    /// </summary>
    /// <param name="g"></param>
    public virtual void DrawTransport(Graphics g)
    {
        if (EntityPlane == null || !_startPosX.HasValue || !_startPosY.HasValue)
        {
            return;
        }

        Pen pen = new(Color.Black);
        Brush additionalBrush = new SolidBrush(EntityPlane.BodyColor);


        //шасси
        g.DrawLine(pen, _startPosX.Value + 50, _startPosY.Value + 55, _startPosX.Value + 50, _startPosY.Value + 70);
        g.DrawLine(pen, _startPosX.Value + 150, _startPosY.Value + 55, _startPosX.Value + 150, _startPosY.Value + 70);
        g.FillEllipse(additionalBrush, _startPosX.Value + 40, _startPosY.Value + 65, 10, 10);
        g.FillEllipse(additionalBrush, _startPosX.Value + 50, _startPosY.Value + 65, 10, 10);
        g.FillEllipse(additionalBrush, _startPosX.Value + 145, _startPosY.Value + 65, 10, 10);
        // корпус
        Brush br = new SolidBrush(EntityPlane.BodyColor);
        g.DrawRectangle(pen, _startPosX.Value, _startPosY.Value + 25, 170, 30);
        g.FillRectangle(br, _startPosX.Value, _startPosY.Value + 25, 170, 30);
        // бак
        Brush blackBrush = new SolidBrush(Color.Black);
        g.FillEllipse(blackBrush, _startPosX.Value + 75, _startPosY.Value + 30, 50, 10);
        // нос самолёта
        Pen blackPen = new Pen(Color.Black, 2);
        Brush blueBrush = new SolidBrush(Color.LightBlue);
        Point point1 = new Point(_startPosX.Value + 170, _startPosY.Value + 25);
        Point point2 = new Point(_startPosX.Value + 200, _startPosY.Value + 40);
        Point point3 = new Point(_startPosX.Value + 170, _startPosY.Value + 55);
        Point[] curvePoints =
         {
                 point1,
                 point2,
                 point3,
             };
        g.FillPolygon(blueBrush, curvePoints);
        g.DrawPolygon(blackPen, curvePoints);
        g.DrawLine(blackPen, _startPosX.Value + 170, _startPosY.Value + 40, _startPosX.Value + 200, _startPosY.Value + 40);
        // хвост
        Brush bw = new SolidBrush(EntityPlane.BodyColor);
        g.FillPolygon(bw, new Point[]
        {
            new Point((int) _startPosX, (int) _startPosY + 40 ), new Point((int) _startPosX + 80, (int) _startPosY + 30),
            new Point((int) _startPosX, (int) _startPosY), new Point((int) _startPosX ,  (int) _startPosY + 40),
        }
        );
    }
}

