using ProjectAirPlane.Drawnings;
using ProjectAirPlane.MovementStrategy;


namespace ProjectAirPlane;

/// <summary>
///     " "
/// </summary>
public partial class FormAirPlane : Form
{
    /// <summary>
    /// -   
    /// </summary>
    private DrawningPlane? _drawningPlane;

    /// <summary>
	///  
	/// </summary>
	private AbstractStrategy? _strategy;

    /// <summary>
    ///  
    /// </summary>
    public FormAirPlane()
    {
        InitializeComponent();
        _strategy = null;
    }

    /// <summary>
    ///   
    /// </summary>
    private void Draw()
    {
        if (_drawningPlane == null)
        {
            return;
        }

        Bitmap bmp = new(pictureBoxAirPlane.Width, pictureBoxAirPlane.Height);
        Graphics gr = Graphics.FromImage(bmp);
        _drawningPlane.DrawTransport(gr);
        pictureBoxAirPlane.Image = bmp;
    }

    /// <summary>
	///   -
	/// </summary>
	/// <param name="type">  </param>
	private void CreateObject(string type)
    {
        Random random = new();
        switch (type)
        {
            case nameof(DrawningPlane):
                _drawningPlane = new DrawningPlane(random.Next(100, 300), random.Next(1000, 3000),
                    Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256)));
                break;
            case nameof(DrawningAirPlane):
                _drawningPlane = new DrawningAirPlane(random.Next(100, 300), random.Next(1000, 3000),
                    Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256)),
                    Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256)),
                    Convert.ToBoolean(random.Next(0, 2)), Convert.ToBoolean(random.Next(0, 2)), Convert.ToBoolean(random.Next(0, 2)));
                break;
            default:
                return;
        }

        _drawningPlane.SetPictureSize(pictureBoxAirPlane.Width, pictureBoxAirPlane.Height);
        _drawningPlane.SetPosition(random.Next(10, 100), random.Next(10, 100));
        _strategy = null;
        comboBoxStrategy.Enabled = true;
        Draw();
    }

    /// <summary>
    ///    "   "
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonCreateAirPlane_Click(object sender, EventArgs e) => CreateObject(nameof(DrawningAirPlane));

    /// <summary>
    ///    " "
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonCreatePlane_Click(object sender, EventArgs e) => CreateObject(nameof(DrawningPlane));



    /// <summary>
    ///     (  )
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonMove_Click(object sender, EventArgs e)
    {
        if (_drawningPlane == null)
        {
            return;
        }

        string name = ((Button)sender)?.Name ?? string.Empty;
        bool result = false;
        switch (name)
        {
            case "buttonUp":
                result = _drawningPlane.MoveTransport(DirectionType.Up);
                break;
            case "buttonDown":
                result = _drawningPlane.MoveTransport(DirectionType.Down);
                break;
            case "buttonLeft":
                result = _drawningPlane.MoveTransport(DirectionType.Left);
                break;
            case "buttonRight":
                result = _drawningPlane.MoveTransport(DirectionType.Right);
                break;
        }

        if (result)
        {
            Draw();
        }
    }

    /// <summary>
	///    ""
	/// </summary>
	/// <param name="sender"></param>
	/// <param name="e"></param>
	private void ButtonStrategyStep_Click(object sender, EventArgs e)
    {
        if (_drawningPlane == null) return;

        if (comboBoxStrategy.Enabled)
        {
            _strategy = comboBoxStrategy.SelectedIndex switch
            {
                0 => new MoveToCenter(),
                1 => new MoveToBorder(),
                _ => null,
            };
            if (_strategy == null)
            {
                return;
            }
            _strategy.SetData(new MoveablePlane(_drawningPlane), pictureBoxAirPlane.Width, pictureBoxAirPlane.Height);
        }

        if (_strategy == null)
        {
            return;
        }

        comboBoxStrategy.Enabled = false;
        _strategy.MakeStep();
        Draw();

        if (_strategy.GetStatus() == StrategyStatus.Finish)
        {
            comboBoxStrategy.Enabled = true;
            _strategy = null;
        }
    }


}